package meerkat.mixer.main;

import meerkat.crypto.mixnet.Mix2ZeroKnowledgeVerifier;
import meerkat.crypto.mixnet.MixerOutput;
import meerkat.protobuf.Crypto;
import meerkat.mixer.necessary.AsyncBulletinBoardClient;
import meerkat.mixer.necessary.CompleteBatch;
import meerkat.mixer.verifier.VerifyTable;

import java.util.Arrays;
import java.util.List;

/**
 * Created by Tzlil on 12/17/2015.
 * implements AsyncBulletinBoardClient.ClientCallback<CompleteBatch>
 */
public class BatchHandler implements AsyncBulletinBoardClient.ClientCallback<CompleteBatch> {

    private MixerOutput mixerOutput;
    private boolean msgReceived;
    private Throwable t;
    private CompleteBatch msg;

    private final int n, layers;
    private final Mix2ZeroKnowledgeVerifier verifier;

    /**
     * constructor
     * @param n
     * @param layers
     * @param verifier
     */
    public BatchHandler(int n, int layers, Mix2ZeroKnowledgeVerifier verifier) {
        this.mixerOutput = null;
        this.n = n;
        this.layers = layers;
        this.msgReceived = false;
        this.t = null;
        this.verifier = verifier;
    }

    @Override
    public void handleCallback(CompleteBatch msg) {
        this.msg = msg;
        synchronized (this) {
            this.msgReceived = true;
            notifyAll();
        }
    }

    @Override
    public void handleFailure(Throwable t) {
        this.t = t;
        synchronized (this) {
            this.msgReceived = true;
            notifyAll();
        }
    }

    /**
     * return true iff msg was received
     * @return msgReceived
     */
    public boolean isMsgReceived() {
        return msgReceived;
    }

    /**
     * convert batch data to meerkat.mixer.mixing output
     * @throws Exception
     */
    private void convertMessage() throws Exception {
        BatchConverter batchConverter = new BatchConverter(n,layers);
        this.mixerOutput = batchConverter.BatchChunkList2MixerOutput(msg.getBatchChunkList());
    }

    /**
     * call convert message, and if succeed verify the table
     * @return return true iff the given batch message is valid meerkat.mixer.mixing output
     * @throws Exception
     */
    public boolean verifyTable() throws Exception {
        if (mixerOutput == null) {
            convertMessage();
        }
        return VerifyTable.verifyTable(verifier, n, mixerOutput);
    }

    /**
     * extract input for meerkat.mixer.mixing from previous mixers output
     * @return last layer of encrypted votes as list
     * @throws Throwable in case if failure
     */
    public List<Crypto.RerandomizableEncryptedMessage> getInputForMixer() throws Throwable {
        if (t != null) {
            throw t;
        }
        if(!verifyTable()){
            throw new Exception("in valid table");
        }
        return Arrays.asList(mixerOutput.getEncryptedMessages()[layers]);//there are layers + 1
    }
}

